@@ -1,8 +1,9 @@
 use vortex_array::compute::{
-    BetweenFn, BetweenOptions, FilterFn, ScalarAtFn, SearchSortedFn, SliceFn, TakeFn, between,
+    BetweenFn, BetweenOptions, FilterKernelAdapter, KernelRef, ScalarAtFn, SearchSortedFn, SliceFn,
+    TakeFn, between,
 };
 use vortex_array::vtable::ComputeVTable;
-use vortex_array::{Array, ArrayRef, IntoArray};
+use vortex_array::{Array, ArrayComputeImpl, ArrayRef, IntoArray};
 use vortex_error::VortexResult;
 
 use crate::{BitPackedArray, BitPackedEncoding};
@@ -13,8 +14,12 @@ mod search_sorted;
 mod slice;
 mod take;
 
+impl ArrayComputeImpl for BitPackedArray {
+    const FILTER: Option<KernelRef> = FilterKernelAdapter(BitPackedEncoding).some();
+}
+
 impl ComputeVTable for BitPackedEncoding {
-    fn filter_fn(&self) -> Option<&dyn FilterFn<&dyn Array>> {
+    fn between_fn(&self) -> Option<&dyn BetweenFn<&dyn Array>> {
         Some(self)
     }
 
@@ -33,10 +38,6 @@ impl ComputeVTable for BitPackedEncoding {
     fn take_fn(&self) -> Option<&dyn TakeFn<&dyn Array>> {
         Some(self)
     }
-
-    fn between_fn(&self) -> Option<&dyn BetweenFn<&dyn Array>> {
-        Some(self)
-    }
 }
 
 fn chunked_indices<F: FnMut(usize, &[usize])>(
@@ -90,3 +91,19 @@ impl BetweenFn<&BitPackedArray> for BitPackedEncoding {
         .map(Some)
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::bitpacking::compute::chunked_indices;
+
+    #[test]
+    fn chunk_indices_repeated() {
+        let mut called = false;
+        chunked_indices([0; 1025].into_iter(), 0, |chunk_idx, idxs| {
+            assert_eq!(chunk_idx, 0);
+            assert_eq!(idxs, [0; 1025]);
+            called = true;
+        });
+        assert!(called);
+    }
+}
