@@ -8,14 +8,14 @@ use vortex_array::patches::Patches;
 use vortex_array::stats::{ArrayStats, StatsSetRef};
 use vortex_array::validity::Validity;
 use vortex_array::variants::PrimitiveArrayTrait;
-use vortex_array::vtable::{EncodingVTable, StatisticsVTable, VTableRef};
+use vortex_array::vtable::VTableRef;
 use vortex_array::{
-    Array, ArrayCanonicalImpl, ArrayExt, ArrayImpl, ArrayStatisticsImpl, ArrayValidityImpl,
-    ArrayVariantsImpl, Canonical, Encoding, EncodingId, RkyvMetadata, try_from_array_ref,
+    Array, ArrayCanonicalImpl, ArrayExt, ArrayImpl, ArrayRef, ArrayStatisticsImpl,
+    ArrayValidityImpl, ArrayVariantsImpl, Canonical, Encoding, RkyvMetadata, try_from_array_ref,
 };
 use vortex_buffer::ByteBuffer;
 use vortex_dtype::{DType, NativePType, PType, match_each_integer_ptype_with_unsigned_type};
-use vortex_error::{VortexExpect as _, VortexResult, vortex_bail, vortex_err};
+use vortex_error::{VortexExpect, VortexResult, vortex_bail, vortex_err};
 use vortex_mask::Mask;
 
 use crate::bitpacking::serde::BitPackedMetadata;
@@ -44,12 +44,6 @@ impl Encoding for BitPackedEncoding {
     type Metadata = RkyvMetadata<BitPackedMetadata>;
 }
 
-impl EncodingVTable for BitPackedEncoding {
-    fn id(&self) -> EncodingId {
-        EncodingId::new_ref("fastlanes.bitpacked")
-    }
-}
-
 /// NB: All non-null values in the patches array are considered patches
 impl BitPackedArray {
     /// Create a new bitpacked array using a buffer of packed data.
@@ -239,6 +233,32 @@ impl ArrayImpl for BitPackedArray {
     fn _vtable(&self) -> VTableRef {
         VTableRef::new_ref(&BitPackedEncoding)
     }
+
+    fn _with_children(&self, children: &[ArrayRef]) -> VortexResult<Self> {
+        let patches = self.patches().map(|existing| {
+            let indices = children[0].clone();
+            let values = children[1].clone();
+            Patches::new(existing.array_len(), existing.offset(), indices, values)
+        });
+
+        let validity = if self.validity().is_array() {
+            Validity::Array(children[children.len() - 1].clone())
+        } else {
+            self.validity().clone()
+        };
+
+        unsafe {
+            Self::new_unchecked_with_offset(
+                self.packed().clone(),
+                self.ptype(),
+                validity,
+                patches,
+                self.bit_width(),
+                self.len(),
+                self.offset(),
+            )
+        }
+    }
 }
 
 impl ArrayCanonicalImpl for BitPackedArray {
@@ -254,10 +274,10 @@ impl ArrayCanonicalImpl for BitPackedArray {
                     .as_any_mut()
                     .downcast_mut()
                     .vortex_expect("bit packed array must canonicalize into a primitive array"),
-                // SAFETY: UnsignedT is the unsigned verison of T, reinterpreting &[UnsignedT] to
+                // SAFETY: UnsignedT is the unsigned version of T, reinterpreting &[UnsignedT] to
                 // &[T] is therefore safe.
                 |x| unsafe { std::mem::transmute(x) },
-                // SAFETY: UnsignedT is the unsigned verison of T, reinterpreting &mut [T] to
+                // SAFETY: UnsignedT is the unsigned version of T, reinterpreting &mut [T] to
                 // &mut [UnsignedT] is therefore safe.
                 |x| unsafe { std::mem::transmute(x) },
             )
@@ -285,7 +305,7 @@ impl ArrayValidityImpl for BitPackedArray {
     }
 
     fn _validity_mask(&self) -> VortexResult<Mask> {
-        self.validity.to_logical(self.len())
+        self.validity.to_mask(self.len())
     }
 }
 
@@ -295,8 +315,6 @@ impl ArrayVariantsImpl for BitPackedArray {
     }
 }
 
-impl StatisticsVTable<&BitPackedArray> for BitPackedEncoding {}
-
 impl PrimitiveArrayTrait for BitPackedArray {}
 
 #[cfg(test)]
