@@ -5,10 +5,10 @@ use vortex_array::arrays::PrimitiveArray;
 use vortex_array::stats::{ArrayStats, StatsSetRef};
 use vortex_array::validity::Validity;
 use vortex_array::variants::PrimitiveArrayTrait;
-use vortex_array::vtable::{EncodingVTable, StatisticsVTable, VTableRef};
+use vortex_array::vtable::VTableRef;
 use vortex_array::{
     Array, ArrayCanonicalImpl, ArrayImpl, ArrayRef, ArrayStatisticsImpl, ArrayValidityImpl,
-    ArrayVariantsImpl, Canonical, Encoding, EncodingId, RkyvMetadata,
+    ArrayVariantsImpl, Canonical, Encoding, RkyvMetadata,
 };
 use vortex_buffer::Buffer;
 use vortex_dtype::{DType, NativePType, PType, match_each_unsigned_integer_ptype};
@@ -38,12 +38,6 @@ impl Encoding for DeltaEncoding {
     type Metadata = RkyvMetadata<DeltaMetadata>;
 }
 
-impl EncodingVTable for DeltaEncoding {
-    fn id(&self) -> EncodingId {
-        EncodingId::new_ref("fastlanes.delta")
-    }
-}
-
 /// A FastLanes-style delta-encoded array of primitive values.
 ///
 /// A [`DeltaArray`] comprises a sequence of _chunks_ each representing 1,024 delta-encoded values,
@@ -228,6 +222,22 @@ impl ArrayImpl for DeltaArray {
     fn _vtable(&self) -> VTableRef {
         VTableRef::new_ref(&DeltaEncoding)
     }
+
+    fn _with_children(&self, children: &[ArrayRef]) -> VortexResult<Self> {
+        let validity = if self.validity().is_array() {
+            Validity::Array(children[2].clone())
+        } else {
+            self.validity().clone()
+        };
+
+        Self::try_new(
+            children[0].clone(),
+            children[1].clone(),
+            validity,
+            self.offset,
+            self.len(),
+        )
+    }
 }
 
 impl ArrayCanonicalImpl for DeltaArray {
@@ -256,7 +266,7 @@ impl ArrayValidityImpl for DeltaArray {
     }
 
     fn _validity_mask(&self) -> VortexResult<Mask> {
-        self.validity.to_logical(self.len)
+        self.validity.to_mask(self.len)
     }
 }
 
@@ -267,5 +277,3 @@ impl ArrayVariantsImpl for DeltaArray {
 }
 
 impl PrimitiveArrayTrait for DeltaArray {}
-
-impl StatisticsVTable<&DeltaArray> for DeltaEncoding {}
