@@ -1,10 +1,10 @@
 use pyo3::prelude::*;
 use vortex::encodings::fastlanes::{BitPackedEncoding, DeltaEncoding, FoREncoding};
 
-use crate::arrays::{AsArrayRef, EncodingSubclass, PyArray};
+use crate::arrays::native::{AsArrayRef, EncodingSubclass, PyNativeArray};
 
 /// Concrete class for arrays with `fastlanes.bitpacked` encoding.
-#[pyclass(name = "FastLanesBitPackedArray", module = "vortex", extends=PyArray, frozen)]
+#[pyclass(name = "FastLanesBitPackedArray", module = "vortex", extends=PyNativeArray, frozen)]
 pub(crate) struct PyFastLanesBitPackedArray;
 
 impl EncodingSubclass for PyFastLanesBitPackedArray {
@@ -13,11 +13,6 @@ impl EncodingSubclass for PyFastLanesBitPackedArray {
 
 #[pymethods]
 impl PyFastLanesBitPackedArray {
-    #[new]
-    fn new(array: Bound<PyArray>) -> PyResult<Bound<Self>> {
-        PyArray::init_encoding(array, &BitPackedEncoding, PyFastLanesBitPackedArray)
-    }
-
     /// Returns the bit width of the packed values.
     #[getter]
     fn bit_width(self_: PyRef<'_, Self>) -> u8 {
@@ -26,33 +21,17 @@ impl PyFastLanesBitPackedArray {
 }
 
 /// Concrete class for arrays with `fastlanes.delta` encoding.
-#[pyclass(name = "FastLanesDeltaArray", module = "vortex", extends=PyArray, frozen)]
+#[pyclass(name = "FastLanesDeltaArray", module = "vortex", extends=PyNativeArray, frozen)]
 pub(crate) struct PyFastLanesDeltaArray;
 
 impl EncodingSubclass for PyFastLanesDeltaArray {
     type Encoding = DeltaEncoding;
 }
 
-#[pymethods]
-impl PyFastLanesDeltaArray {
-    #[new]
-    fn new(array: Bound<PyArray>) -> PyResult<Bound<Self>> {
-        PyArray::init_encoding(array, &DeltaEncoding, PyFastLanesDeltaArray)
-    }
-}
-
 /// Concrete class for arrays with `fastlanes.for` encoding.
-#[pyclass(name = "FastLanesFoRArray", module = "vortex", extends=PyArray, frozen)]
+#[pyclass(name = "FastLanesFoRArray", module = "vortex", extends=PyNativeArray, frozen)]
 pub(crate) struct PyFastLanesFoRArray;
 
 impl EncodingSubclass for PyFastLanesFoRArray {
     type Encoding = FoREncoding;
 }
-
-#[pymethods]
-impl PyFastLanesFoRArray {
-    #[new]
-    fn new(array: Bound<PyArray>) -> PyResult<Bound<Self>> {
-        PyArray::init_encoding(array, &FoREncoding, PyFastLanesFoRArray)
-    }
-}
