@@ -2,38 +2,31 @@ pub(crate) mod builtins;
 pub(crate) mod compressed;
 pub(crate) mod fastlanes;
 pub(crate) mod from_arrow;
+mod native;
 pub(crate) mod py;
-pub(crate) mod typed;
 
-use std::ops::Deref;
+use std::sync::Arc;
 
 use arrow::array::{Array as ArrowArray, ArrayRef as ArrowArrayRef};
 use arrow::pyarrow::ToPyArrow;
-use pyo3::PyClass;
-use pyo3::exceptions::PyValueError;
+use pyo3::exceptions::{PyTypeError, PyValueError};
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList};
 use vortex::arrays::ChunkedArray;
-use vortex::arrow::{IntoArrowArray, infer_data_type};
+use vortex::arrow::IntoArrowArray;
 use vortex::compute::{Operator, compare, fill_forward, scalar_at, slice, take};
-use vortex::dtype::{DType, PType};
-use vortex::error::VortexExpect;
+use vortex::error::VortexError;
 use vortex::mask::Mask;
 use vortex::nbytes::NBytes;
-use vortex::vtable::EncodingVTable;
-use vortex::{Array, ArrayExt, ArrayRef, Encoding};
+use vortex::{Array, ArrayExt, ArrayRef};
 
-use crate::arrays::typed::{
-    PyBinaryTypeArray, PyBoolTypeArray, PyExtensionTypeArray, PyFloat16TypeArray,
-    PyFloat32TypeArray, PyFloat64TypeArray, PyFloatTypeArray, PyInt8TypeArray, PyInt16TypeArray,
-    PyInt32TypeArray, PyInt64TypeArray, PyIntTypeArray, PyIntegerTypeArray, PyListTypeArray,
-    PyNullTypeArray, PyPrimitiveTypeArray, PyStructTypeArray, PyUInt8TypeArray, PyUInt16TypeArray,
-    PyUInt32TypeArray, PyUInt64TypeArray, PyUIntTypeArray, PyUtf8TypeArray,
-};
+use crate::arrays::native::PyNativeArray;
+use crate::arrays::py::PyArrayInstance;
 use crate::dtype::PyDType;
-use crate::install_module;
 use crate::python_repr::PythonRepr;
 use crate::scalar::PyScalar;
+use crate::serde::context::PyArrayContext;
+use crate::{PyVortex, install_module};
 
 pub(crate) fn init(py: Python, parent: &Bound<PyModule>) -> PyResult<()> {
     let m = PyModule::new(py, "arrays")?;
@@ -41,10 +34,9 @@ pub(crate) fn init(py: Python, parent: &Bound<PyModule>) -> PyResult<()> {
     install_module("vortex._lib.arrays", &m)?;
 
     m.add_class::<PyArray>()?;
+    m.add_class::<PyNativeArray>()?;
 
     // Canonical encodings
-    m.add_class::<builtins::PyConstantArray>()?;
-    m.add_class::<builtins::PyChunkedArray>()?;
     m.add_class::<builtins::PyNullArray>()?;
     m.add_class::<builtins::PyBoolArray>()?;
     m.add_class::<builtins::PyPrimitiveArray>()?;
@@ -54,6 +46,11 @@ pub(crate) fn init(py: Python, parent: &Bound<PyModule>) -> PyResult<()> {
     m.add_class::<builtins::PyListArray>()?;
     m.add_class::<builtins::PyExtensionArray>()?;
 
+    // Utility encodings
+    m.add_class::<builtins::PyConstantArray>()?;
+    m.add_class::<builtins::PyChunkedArray>()?;
+    m.add_class::<builtins::PyByteBoolArray>()?;
+
     // Compressed encodings
     m.add_class::<compressed::PyAlpArray>()?;
     m.add_class::<compressed::PyAlpRdArray>()?;
@@ -69,37 +66,40 @@ pub(crate) fn init(py: Python, parent: &Bound<PyModule>) -> PyResult<()> {
     m.add_class::<fastlanes::PyFastLanesDeltaArray>()?;
     m.add_class::<fastlanes::PyFastLanesFoRArray>()?;
 
-    // Pluggable encodings
-    m.add_class::<py::PyEncoding>()?;
-
-    // Typed arrays
-    m.add_class::<PyNullTypeArray>()?;
-    m.add_class::<PyBoolTypeArray>()?;
-    m.add_class::<PyPrimitiveTypeArray>()?;
-    m.add_class::<PyIntegerTypeArray>()?;
-    m.add_class::<PyUIntTypeArray>()?;
-    m.add_class::<PyIntTypeArray>()?;
-    m.add_class::<PyFloatTypeArray>()?;
-    m.add_class::<PyUInt8TypeArray>()?;
-    m.add_class::<PyUInt16TypeArray>()?;
-    m.add_class::<PyUInt32TypeArray>()?;
-    m.add_class::<PyUInt64TypeArray>()?;
-    m.add_class::<PyInt8TypeArray>()?;
-    m.add_class::<PyInt16TypeArray>()?;
-    m.add_class::<PyInt32TypeArray>()?;
-    m.add_class::<PyInt64TypeArray>()?;
-    m.add_class::<PyFloat16TypeArray>()?;
-    m.add_class::<PyFloat32TypeArray>()?;
-    m.add_class::<PyFloat64TypeArray>()?;
-    m.add_class::<PyUtf8TypeArray>()?;
-    m.add_class::<PyBinaryTypeArray>()?;
-    m.add_class::<PyStructTypeArray>()?;
-    m.add_class::<PyListTypeArray>()?;
-    m.add_class::<PyExtensionTypeArray>()?;
-
     Ok(())
 }
 
+/// A type adapter used to extract an ArrayRef from a Python object.
+pub type PyArrayRef = PyVortex<ArrayRef>;
+
+impl<'py> FromPyObject<'py> for PyArrayRef {
+    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+        // If it's already native, then we're done.
+        if let Ok(native) = ob.downcast::<PyNativeArray>() {
+            return Ok(Self(native.get().inner().clone()));
+        }
+
+        // Otherwise, if it's a subclass of `PyArray`, then we can extract the inner array.
+        PyArrayInstance::extract_bound(ob).map(|instance| Self(Arc::new(instance)))
+    }
+}
+
+impl<'py> IntoPyObject<'py> for PyArrayRef {
+    type Target = PyAny;
+    type Output = Bound<'py, PyAny>;
+    type Error = VortexError;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        // If the ArrayRef is a PyArrayInstance, extract the Python object.
+        if let Some(pyarray) = self.0.as_any().downcast_ref::<PyArrayInstance>() {
+            return pyarray.clone().into_pyobject(py);
+        }
+
+        // Otherwise, wrap the ArrayRef in a PyNativeArray.
+        Ok(PyNativeArray::init(py, self.0.clone())?.into_any())
+    }
+}
+
 /// An array of zero or more *rows* each with the same set of *columns*.
 ///
 /// Examples
@@ -165,149 +165,17 @@ pub(crate) fn init(py: Python, parent: &Bound<PyModule>) -> PyResult<()> {
 ///        true
 ///     ]
 #[pyclass(name = "Array", module = "vortex", sequence, subclass, frozen)]
-#[derive(Clone)]
-pub struct PyArray(ArrayRef);
-
-impl Deref for PyArray {
-    type Target = ArrayRef;
-
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
-
-impl PyArray {
-    /// Initialize a [`PyArray`] from a Vortex [`ArrayRef`], ensuring we return the correct typed
-    /// subclass array.
-    pub fn init(py: Python, array: ArrayRef) -> PyResult<Bound<PyArray>> {
-        fn unsigned(array: ArrayRef) -> PyClassInitializer<PyUIntTypeArray> {
-            PyClassInitializer::from(PyArray(array))
-                .add_subclass(PyPrimitiveTypeArray)
-                .add_subclass(PyIntegerTypeArray)
-                .add_subclass(PyUIntTypeArray)
-        }
-
-        fn signed(array: ArrayRef) -> PyClassInitializer<PyIntTypeArray> {
-            PyClassInitializer::from(PyArray(array))
-                .add_subclass(PyPrimitiveTypeArray)
-                .add_subclass(PyIntegerTypeArray)
-                .add_subclass(PyIntTypeArray)
-        }
-
-        fn float(array: ArrayRef) -> PyClassInitializer<PyFloatTypeArray> {
-            PyClassInitializer::from(PyArray(array))
-                .add_subclass(PyPrimitiveTypeArray)
-                .add_subclass(PyFloatTypeArray)
-        }
-
-        match array.dtype() {
-            DType::Null => Self::with_subclass(py, array, PyNullTypeArray),
-            DType::Bool(_) => Self::with_subclass(py, array, PyBoolTypeArray),
-            DType::Primitive(ptype, _) => match ptype {
-                PType::U8 => Self::with_subclass_initializer(
-                    py,
-                    unsigned(array).add_subclass(PyUInt8TypeArray),
-                ),
-                PType::U16 => Self::with_subclass_initializer(
-                    py,
-                    unsigned(array).add_subclass(PyUInt16TypeArray),
-                ),
-                PType::U32 => Self::with_subclass_initializer(
-                    py,
-                    unsigned(array).add_subclass(PyUInt32TypeArray),
-                ),
-                PType::U64 => Self::with_subclass_initializer(
-                    py,
-                    unsigned(array).add_subclass(PyUInt64TypeArray),
-                ),
-                PType::I8 => {
-                    Self::with_subclass_initializer(py, signed(array).add_subclass(PyInt8TypeArray))
-                }
-                PType::I16 => Self::with_subclass_initializer(
-                    py,
-                    signed(array).add_subclass(PyInt16TypeArray),
-                ),
-                PType::I32 => Self::with_subclass_initializer(
-                    py,
-                    signed(array).add_subclass(PyInt32TypeArray),
-                ),
-                PType::I64 => Self::with_subclass_initializer(
-                    py,
-                    signed(array).add_subclass(PyInt64TypeArray),
-                ),
-                PType::F16 => Self::with_subclass_initializer(
-                    py,
-                    float(array).add_subclass(PyFloat16TypeArray),
-                ),
-                PType::F32 => Self::with_subclass_initializer(
-                    py,
-                    float(array).add_subclass(PyFloat32TypeArray),
-                ),
-                PType::F64 => Self::with_subclass_initializer(
-                    py,
-                    float(array).add_subclass(PyFloat64TypeArray),
-                ),
-            },
-            DType::Utf8(_) => Self::with_subclass(py, array, PyUtf8TypeArray),
-            DType::Binary(_) => Self::with_subclass(py, array, PyBinaryTypeArray),
-            DType::Struct(..) => Self::with_subclass(py, array, PyStructTypeArray),
-            DType::List(..) => Self::with_subclass(py, array, PyListTypeArray),
-            DType::Extension(_) => Self::with_subclass(py, array, PyExtensionTypeArray),
-        }
-    }
-
-    /// Initialize a [`PyArray`] with an [`EncodingSubclass`].
-    pub fn init_encoding<'py, S: EncodingSubclass>(
-        array: Bound<'py, PyArray>,
-        encoding: &'static <S as EncodingSubclass>::Encoding,
-        subclass: S,
-    ) -> PyResult<Bound<'py, S>> {
-        if array.get().deref().encoding() != encoding.id() {
-            return Err(PyValueError::new_err(format!(
-                "Array has encoding {}, expected {}",
-                array.get().deref().encoding(),
-                encoding.id()
-            )));
-        }
-        Bound::new(
-            array.py(),
-            PyClassInitializer::from(array.get().clone()).add_subclass(subclass),
-        )
-    }
-
-    fn with_subclass<S: PyClass<BaseType = PyArray>>(
-        py: Python,
-        array: ArrayRef,
-        subclass: S,
-    ) -> PyResult<Bound<PyArray>> {
-        Ok(Bound::new(
-            py,
-            PyClassInitializer::from(PyArray(array)).add_subclass(subclass),
-        )?
-        .into_any()
-        .downcast_into::<PyArray>()?)
-    }
-
-    fn with_subclass_initializer<S: PyClass>(
-        py: Python,
-        intializer: PyClassInitializer<S>,
-    ) -> PyResult<Bound<PyArray>> {
-        Ok(Bound::new(py, intializer)?
-            .into_any()
-            .downcast_into::<PyArray>()?)
-    }
-
-    pub fn inner(&self) -> &ArrayRef {
-        &self.0
-    }
-
-    pub fn into_inner(self) -> ArrayRef {
-        self.0
-    }
-}
+pub struct PyArray;
 
 #[pymethods]
 impl PyArray {
+    #[new]
+    #[pyo3(signature = (*args, **kwargs))]
+    #[allow(unused_variables)]
+    fn new(args: &Bound<'_, PyAny>, kwargs: Option<&Bound<'_, PyAny>>) -> Self {
+        Self
+    }
+
     /// Convert a PyArrow object into a Vortex array.
     ///
     /// One of :class:`pyarrow.Array`, :class:`pyarrow.ChunkedArray`, or :class:`pyarrow.Table`.
@@ -316,7 +184,7 @@ impl PyArray {
     /// -------
     /// :class:`~vortex.Array`
     #[staticmethod]
-    fn from_arrow(obj: Bound<'_, PyAny>) -> PyResult<Bound<'_, PyArray>> {
+    fn from_arrow(obj: Bound<'_, PyAny>) -> PyResult<PyArrayRef> {
         from_arrow::from_arrow(&obj)
     }
 
@@ -344,26 +212,23 @@ impl PyArray {
     ///       2,
     ///       3
     ///     ]
-    fn to_arrow_array(self_: PyRef<'_, Self>) -> PyResult<Bound<PyAny>> {
+    fn to_arrow_array<'py>(self_: &'py Bound<'py, Self>) -> PyResult<Bound<'py, PyAny>> {
         // NOTE(ngates): for struct arrays, we could also return a RecordBatchStreamReader.
+        let array = PyArrayRef::extract_bound(self_.as_any())?.into_inner();
         let py = self_.py();
-        let vortex = &self_.0;
 
-        if let Some(chunked_array) = vortex.as_opt::<ChunkedArray>() {
+        if let Some(chunked_array) = array.as_opt::<ChunkedArray>() {
             // We figure out a single Arrow Data Type to convert all chunks into, otherwise
             // the preferred type of each chunk may be different.
-            let arrow_dtype = infer_data_type(chunked_array.dtype())?;
+            let arrow_dtype = chunked_array.dtype().to_arrow_dtype()?;
 
             let chunks = chunked_array
                 .chunks()
                 .iter()
                 .map(|chunk| PyResult::Ok(chunk.clone().into_arrow(&arrow_dtype)?))
                 .collect::<PyResult<Vec<ArrowArrayRef>>>()?;
-            if chunks.is_empty() {
-                return Err(PyValueError::new_err("No chunks in array"));
-            }
 
-            let pa_data_type = chunks[0].data_type().clone().to_pyarrow(py)?;
+            let pa_data_type = arrow_dtype.clone().to_pyarrow(py)?;
             let chunks = chunks
                 .iter()
                 .map(|arrow_array| arrow_array.into_data().to_pyarrow(py))
@@ -379,7 +244,7 @@ impl PyArray {
                 Some(&kwargs),
             )
         } else {
-            Ok(vortex
+            Ok(array
                 .clone()
                 .into_arrow_preferred()?
                 .into_data()
@@ -388,24 +253,26 @@ impl PyArray {
         }
     }
 
-    fn __len__(&self) -> usize {
-        self.0.len()
+    fn __len__(&self) -> PyResult<usize> {
+        Err(PyTypeError::new_err("__len__ is not implemented for Array"))
     }
 
-    fn __str__(&self) -> String {
-        format!("{}", self.0)
+    fn __str__(&self) -> PyResult<String> {
+        Err(PyTypeError::new_err("__str__ is not implemented for Array"))
     }
 
     /// Returns the encoding ID of this array.
     #[getter]
-    fn encoding(&self) -> String {
-        self.0.encoding().to_string()
+    fn id(slf: &Bound<Self>) -> PyResult<String> {
+        Ok(PyArrayRef::extract_bound(slf.as_any())?
+            .encoding()
+            .to_string())
     }
 
     /// Returns the number of bytes used by this array.
     #[getter]
-    fn nbytes(&self) -> usize {
-        self.0.nbytes()
+    fn nbytes(slf: &Bound<Self>) -> PyResult<usize> {
+        Ok(PyArrayRef::extract_bound(slf.as_any())?.nbytes())
     }
 
     /// Returns the data type of this array.
@@ -433,50 +300,53 @@ impl PyArray {
     ///     >>> vx.array(['hello, ', 'is', 'it', 'me?']).dtype
     ///     utf8(nullable=False)
     #[getter]
-    fn dtype(self_: PyRef<Self>) -> PyResult<Bound<PyDType>> {
-        PyDType::init(self_.py(), self_.0.dtype().clone())
+    fn dtype<'py>(slf: &'py Bound<'py, Self>) -> PyResult<Bound<'py, PyDType>> {
+        PyDType::init(
+            slf.py(),
+            PyArrayRef::extract_bound(slf.as_any())?.dtype().clone(),
+        )
     }
 
     ///Rust docs are *not* copied into Python for __lt__: https://github.com/PyO3/pyo3/issues/4326
-    fn __lt__(&self, other: &Bound<PyArray>) -> PyResult<PyArray> {
-        let other = other.borrow();
-        let inner = compare(&self.0, &other.0, Operator::Lt)?;
-        Ok(PyArray(inner))
+    fn __lt__(slf: Bound<Self>, other: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = compare(&slf, &*other, Operator::Lt)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     ///Rust docs are *not* copied into Python for __le__: https://github.com/PyO3/pyo3/issues/4326
-    fn __le__(&self, other: &Bound<PyArray>) -> PyResult<PyArray> {
-        let other = other.borrow();
-        let inner = compare(&self.0, &other.0, Operator::Lte)?;
-        Ok(PyArray(inner))
+    fn __le__(slf: Bound<Self>, other: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = compare(&*slf, &*other, Operator::Lte)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     ///Rust docs are *not* copied into Python for __eq__: https://github.com/PyO3/pyo3/issues/4326
-    fn __eq__(&self, other: &Bound<PyArray>) -> PyResult<PyArray> {
-        let other = other.borrow();
-        let inner = compare(&self.0, &other.0, Operator::Eq)?;
-        Ok(PyArray(inner))
+    fn __eq__(slf: Bound<Self>, other: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = compare(&*slf, &*other, Operator::Eq)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     ///Rust docs are *not* copied into Python for __ne__: https://github.com/PyO3/pyo3/issues/4326
-    fn __ne__(&self, other: &Bound<PyArray>) -> PyResult<PyArray> {
-        let other = other.borrow();
-        let inner = compare(&self.0, &other.0, Operator::NotEq)?;
-        Ok(PyArray(inner))
+    fn __ne__(slf: Bound<Self>, other: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = compare(&*slf, &*other, Operator::NotEq)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     ///Rust docs are *not* copied into Python for __ge__: https://github.com/PyO3/pyo3/issues/4326
-    fn __ge__(&self, other: &Bound<PyArray>) -> PyResult<PyArray> {
-        let other = other.borrow();
-        let inner = compare(&self.0, &other.0, Operator::Gte)?;
-        Ok(PyArray(inner))
+    fn __ge__(slf: Bound<Self>, other: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = compare(&*slf, &*other, Operator::Gte)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     ///Rust docs are *not* copied into Python for __gt__: https://github.com/PyO3/pyo3/issues/4326
-    fn __gt__(&self, other: &Bound<PyArray>) -> PyResult<PyArray> {
-        let other = other.borrow();
-        let inner = compare(&self.0, &other.0, Operator::Gt)?;
-        Ok(PyArray(inner))
+    fn __gt__(slf: Bound<Self>, other: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = compare(&*slf, &*other, Operator::Gt)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     /// Filter an Array by another Boolean array.
@@ -505,10 +375,10 @@ impl PyArray {
     ///       9,
     ///       5
     ///     ]
-    fn filter(&self, mask: &Bound<PyArray>) -> PyResult<PyArray> {
-        let mask = mask.borrow();
-        let inner = vortex::compute::filter(&self.0, &Mask::try_from(mask.0.as_ref())?)?;
-        Ok(PyArray(inner))
+    fn filter(slf: Bound<Self>, mask: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = vortex::compute::filter(&*slf, &Mask::try_from(&*mask as &dyn Array)?)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     /// Fill forward non-null values over runs of nulls.
@@ -550,9 +420,10 @@ impl PyArray {
     ///       30.07,
     ///       30.07
     ///     ]
-    fn fill_forward(&self) -> PyResult<PyArray> {
-        let inner = fill_forward(&self.0)?;
-        Ok(PyArray(inner))
+    fn fill_forward(slf: Bound<Self>) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = fill_forward(&slf)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     /// Retrieve a row by its index.
@@ -614,8 +485,10 @@ impl PyArray {
     ///     ...
     ///     OverflowError: can't convert negative int to unsigned
     // TODO(ngates): return a vortex.Scalar
-    fn scalar_at(self_: PyRef<'_, Self>, index: usize) -> PyResult<Bound<PyScalar>> {
-        PyScalar::init(self_.py(), scalar_at(&self_.0, index)?)
+    fn scalar_at(slf: Bound<Self>, index: usize) -> PyResult<Bound<PyScalar>> {
+        let py = slf.py();
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        PyScalar::init(py, scalar_at(&slf, index)?)
     }
 
     /// Filter, permute, and/or repeat elements by their index.
@@ -655,8 +528,8 @@ impl PyArray {
     ///       "b",
     ///       "a"
     ///     ]
-    fn take(&self, indices: &Bound<PyArray>) -> PyResult<PyArray> {
-        let indices = &indices.borrow().0;
+    fn take(slf: Bound<Self>, indices: PyArrayRef) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
 
         if !indices.dtype().is_int() {
             return Err(PyValueError::new_err(format!(
@@ -665,8 +538,9 @@ impl PyArray {
             )));
         }
 
-        let inner = take(&self.0, indices)?;
-        Ok(PyArray(inner))
+        let inner = take(&slf, &*indices)?;
+
+        Ok(PyArrayRef::from(inner))
     }
 
     /// Slice this array.
@@ -720,9 +594,10 @@ impl PyArray {
     ///     ...
     ///     OverflowError: can't convert negative int to unsigned
     #[pyo3(signature = (start, end))]
-    fn slice(&self, start: usize, end: usize) -> PyResult<PyArray> {
-        let inner = slice(&self.0, start, end)?;
-        Ok(PyArray(inner))
+    fn slice(slf: Bound<Self>, start: usize, end: usize) -> PyResult<PyArrayRef> {
+        let slf = PyArrayRef::extract_bound(slf.as_any())?.into_inner();
+        let inner = slice(&slf, start, end)?;
+        Ok(PyArrayRef::from(inner))
     }
 
     /// Internal technical details about the encoding of this Array.
@@ -745,34 +620,26 @@ impl PyArray {
     ///     >>> print(arr.tree_display())
     ///     root: vortex.primitive(i64?, len=4) nbytes=33 B (100.00%)
     ///       metadata: EmptyMetadata
-    ///       buffer (align=8): 32 B
+    ///       buffer (align=8): 32 B (96.97%)
     ///       validity: vortex.bool(bool, len=4) nbytes=1 B (3.03%)
     ///         metadata: BoolMetadata { offset: 0 }
-    ///         buffer (align=1): 1 B
+    ///         buffer (align=1): 1 B (100.00%)
     ///     <BLANKLINE>
     ///
     /// Compressed arrays often have more complex, deeply nested encoding trees.
-    fn tree_display(&self) -> String {
-        self.0.tree_display().to_string()
+    fn tree_display(slf: &Bound<Self>) -> PyResult<String> {
+        Ok(PyArrayRef::extract_bound(slf.as_any())?
+            .tree_display()
+            .to_string())
     }
-}
 
-/// A marker trait indicating a PyO3 class is a subclass of Vortex `Array`.
-pub trait EncodingSubclass: PyClass<BaseType = PyArray> {
-    type Encoding: Encoding;
-}
-
-/// Unwrap a downcasted Vortex array from a `PyRef<ArraySubclass>`.
-pub trait AsArrayRef<T> {
-    fn as_array_ref(&self) -> &T;
-}
-
-impl<A: EncodingSubclass> AsArrayRef<<A::Encoding as Encoding>::Array> for PyRef<'_, A> {
-    fn as_array_ref(&self) -> &<A::Encoding as Encoding>::Array {
-        self.as_super()
-            .inner()
-            .as_any()
-            .downcast_ref::<<A::Encoding as Encoding>::Array>()
-            .vortex_expect("Failed to downcast array")
+    fn serialize(slf: &Bound<Self>, ctx: &PyArrayContext) -> PyResult<Vec<Vec<u8>>> {
+        // FIXME(ngates): do not copy to vec, use buffer protocol
+        let array = PyArrayRef::extract_bound(slf.as_any())?;
+        Ok(array
+            .serialize(ctx, &Default::default())
+            .into_iter()
+            .map(|buffer| buffer.to_vec())
+            .collect())
     }
 }
