@@ -1,27 +1,34 @@
 #![allow(unsafe_op_in_unsafe_fn)]
 
+use std::ops::Deref;
 use std::sync::LazyLock;
 
 use pyo3::exceptions::PyRuntimeError;
 use pyo3::prelude::*;
 
-mod arrays;
+pub(crate) mod arrays;
 mod compress;
 mod dataset;
-mod dtype;
+pub(crate) mod dtype;
 mod expr;
+mod file;
 mod io;
+mod iter;
 mod object_store_urls;
 mod python_repr;
 mod record_batch_reader;
 mod registry;
 pub(crate) mod scalar;
+mod serde;
 
 use log::LevelFilter;
 use pyo3_log::{Caching, Logger};
 use tokio::runtime::Runtime;
 use vortex::error::{VortexError, VortexExpect as _};
 
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
 pub static TOKIO_RUNTIME: LazyLock<Runtime> = LazyLock::new(|| {
     Runtime::new()
         .map_err(VortexError::IOError)
@@ -46,10 +53,12 @@ fn _lib(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     dataset::init(py, m)?;
     dtype::init(py, m)?;
     expr::init(py, m)?;
+    file::init(py, m)?;
     io::init(py, m)?;
+    iter::init(py, m)?;
+    registry::init(py, m)?;
     scalar::init(py, m)?;
-
-    m.add_function(wrap_pyfunction!(registry::register, m)?)?;
+    serde::init(py, m)?;
 
     Ok(())
 }
@@ -89,3 +98,27 @@ pub fn install_module(name: &str, module: &Bound<PyModule>) -> PyResult<()> {
 
 /// An adapter struct used to localize trait impls to this crate.
 pub struct PyVortex<T>(pub T);
+
+impl<T> From<T> for PyVortex<T> {
+    fn from(value: T) -> Self {
+        Self(value)
+    }
+}
+
+impl<T> PyVortex<T> {
+    pub fn into_inner(self) -> T {
+        self.0
+    }
+
+    pub fn inner(&self) -> &T {
+        &self.0
+    }
+}
+
+impl<T> Deref for PyVortex<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
