@@ -150,8 +150,8 @@ fn arrow_numeric(
     let nullable = lhs.dtype().is_nullable() || rhs.dtype().is_nullable();
     let len = lhs.len();
 
-    let left = Datum::try_new(lhs.to_array())?;
-    let right = Datum::try_new(rhs.to_array())?;
+    let left = Datum::try_new(lhs)?;
+    let right = Datum::try_new(rhs)?;
 
     let array = match operator {
         BinaryNumericOperator::Add => arrow_arith::numeric::add(&left, &right)?,
@@ -171,13 +171,13 @@ fn arrow_numeric(
 
 #[inline(always)]
 fn check_numeric_result(result: ArrayRef, lhs: &dyn Array, rhs: &dyn Array) -> ArrayRef {
-    debug_assert_eq!(
+    assert_eq!(
         result.len(),
         lhs.len(),
         "Numeric operation length mismatch {}",
         rhs.encoding()
     );
-    debug_assert_eq!(
+    assert_eq!(
         result.dtype(),
         &DType::Primitive(
             PType::try_from(lhs.dtype())
@@ -190,98 +190,6 @@ fn check_numeric_result(result: ArrayRef, lhs: &dyn Array, rhs: &dyn Array) -> A
     result
 }
 
-#[cfg(feature = "test-harness")]
-pub mod test_harness {
-    use num_traits::Num;
-    use vortex_dtype::NativePType;
-    use vortex_error::{VortexResult, vortex_err};
-    use vortex_scalar::{BinaryNumericOperator, PrimitiveScalar, Scalar};
-
-    use crate::arrays::ConstantArray;
-    use crate::compute::{binary_numeric, scalar_at};
-    use crate::{Array, ArrayRef};
-
-    #[allow(clippy::unwrap_used)]
-    fn to_vec_of_scalar(array: &dyn Array) -> Vec<Scalar> {
-        // Not fast, but obviously correct
-        (0..array.len())
-            .map(|index| scalar_at(array, index))
-            .collect::<VortexResult<Vec<_>>>()
-            .unwrap()
-    }
-
-    #[allow(clippy::unwrap_used)]
-    pub fn test_binary_numeric<T: NativePType + Num + Copy>(array: ArrayRef)
-    where
-        Scalar: From<T>,
-    {
-        let canonicalized_array = array.to_canonical().unwrap().into_primitive().unwrap();
-        let original_values = to_vec_of_scalar(&canonicalized_array.into_array());
-
-        let one = T::from(1)
-            .ok_or_else(|| vortex_err!("could not convert 1 into array native type"))
-            .unwrap();
-        let scalar_one = Scalar::from(one).cast(array.dtype()).unwrap();
-
-        let operators: [BinaryNumericOperator; 6] = [
-            BinaryNumericOperator::Add,
-            BinaryNumericOperator::Sub,
-            BinaryNumericOperator::RSub,
-            BinaryNumericOperator::Mul,
-            BinaryNumericOperator::Div,
-            BinaryNumericOperator::RDiv,
-        ];
-
-        for operator in operators {
-            assert_eq!(
-                to_vec_of_scalar(
-                    &binary_numeric(
-                        &array,
-                        &ConstantArray::new(scalar_one.clone(), array.len()).into_array(),
-                        operator
-                    )
-                    .unwrap()
-                ),
-                original_values
-                    .iter()
-                    .map(|x| x
-                        .as_primitive()
-                        .checked_binary_numeric(&scalar_one.as_primitive(), operator)
-                        .unwrap())
-                    .map(<Scalar as From<PrimitiveScalar<'_>>>::from)
-                    .collect::<Vec<Scalar>>(),
-                "({:?}) {} (Constant array of {}) did not produce expected results",
-                array,
-                operator,
-                scalar_one,
-            );
-
-            assert_eq!(
-                to_vec_of_scalar(
-                    &binary_numeric(
-                        &ConstantArray::new(scalar_one.clone(), array.len()).into_array(),
-                        &array,
-                        operator
-                    )
-                    .unwrap()
-                ),
-                original_values
-                    .iter()
-                    .map(|x| scalar_one
-                        .as_primitive()
-                        .checked_binary_numeric(&x.as_primitive(), operator)
-                        .unwrap())
-                    .map(<Scalar as From<PrimitiveScalar<'_>>>::from)
-                    .collect::<Vec<_>>(),
-                "(Constant array of {}) {} ({:?}) did not produce expected results",
-                scalar_one,
-                operator,
-                array,
-            );
-        }
-    }
-}
-
 #[cfg(test)]
 mod test {
     use vortex_buffer::buffer;
