@@ -68,13 +68,13 @@ pub fn slice(array: &dyn Array, start: usize, stop: usize) -> VortexResult<Array
         }
     }
 
-    debug_assert_eq!(
+    assert_eq!(
         sliced.len(),
         stop - start,
         "Slice length mismatch {}",
         array.encoding()
     );
-    debug_assert_eq!(
+    assert_eq!(
         sliced.dtype(),
         array.dtype(),
         "Slice dtype mismatch {}",
@@ -89,8 +89,8 @@ fn derive_sliced_stats(arr: &dyn Array) -> StatsSet {
 
     // an array that is not constant can become constant after slicing
     let is_constant = stats.get_as::<bool>(Stat::IsConstant);
-    let is_sorted = stats.get_as::<bool>(Stat::IsConstant);
-    let is_strict_sorted = stats.get_as::<bool>(Stat::IsConstant);
+    let is_sorted = stats.get_as::<bool>(Stat::IsSorted);
+    let is_strict_sorted = stats.get_as::<bool>(Stat::IsStrictSorted);
 
     let mut stats = stats.keep_inexact_stats(&[
         Stat::Max,
