@@ -5,9 +5,10 @@ use std::ops::{Deref, DerefMut};
 
 use bytes::buf::UninitSlice;
 use bytes::{Buf, BufMut, BytesMut};
-use vortex_error::{vortex_panic, VortexExpect};
+use vortex_error::{VortexExpect, vortex_panic};
 
 use crate::debug::TruncatedDebug;
+use crate::spec_extend::SpecExtend;
 use crate::{Alignment, Buffer, ByteBufferMut};
 
 /// A mutable buffer that maintains a runtime-defined alignment through resizing operations.
@@ -292,7 +293,8 @@ impl<T> BufferMut<T> {
         let mut dst: *mut T = self.bytes.spare_capacity_mut().as_mut_ptr().cast();
         // SAFETY: we checked the capacity in the reserve call
         unsafe {
-            for _ in 0..n {
+            let end = dst.add(n);
+            while dst < end {
                 dst.write(item);
                 dst = dst.add(1);
             }
@@ -413,31 +415,19 @@ impl<T> AsMut<[T]> for BufferMut<T> {
 }
 
 impl<T> Extend<T> for BufferMut<T> {
+    #[inline]
     fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) {
-        let mut iterator = iter.into_iter();
+        <Self as SpecExtend<T, I::IntoIter>>::spec_extend(self, iter.into_iter())
+    }
+}
 
-        // Attempt to reserve enough memory up-front, although this is only a lower bound.
-        let (lower, _upper) = iterator.size_hint();
-        self.reserve(lower);
-
-        let remaining = self.capacity() - self.len();
-
-        let dst: *mut T = self.bytes.spare_capacity_mut().as_mut_ptr().cast();
-        let mut consumed = 0;
-        while consumed < remaining {
-            if let Some(item) = iterator.next() {
-                // SAFETY: We know we have enough capacity to write the item.
-                unsafe { dst.add(consumed).write(item) };
-                consumed += 1;
-            } else {
-                break;
-            }
-        }
-
-        self.length += consumed;
-        unsafe { self.bytes.set_len(self.length * size_of::<T>()) };
-
-        iterator.for_each(|item| self.push(item));
+impl<'a, T> Extend<&'a T> for BufferMut<T>
+where
+    T: Copy + 'a,
+{
+    #[inline]
+    fn extend<I: IntoIterator<Item = &'a T>>(&mut self, iter: I) {
+        self.spec_extend(iter.into_iter())
     }
 }
 
@@ -554,7 +544,7 @@ impl AlignedBytesMut for BytesMut {
 mod test {
     use bytes::{Buf, BufMut};
 
-    use crate::{buffer_mut, Alignment, BufferMut, ByteBufferMut};
+    use crate::{Alignment, BufferMut, ByteBufferMut, buffer_mut};
 
     #[test]
     fn capacity() {
