@@ -2,12 +2,12 @@ use flatbuffers::Follow;
 use vortex_error::{VortexError, vortex_err};
 use vortex_flatbuffers::{FlatBufferRoot, ReadFlatBuffer, WriteFlatBuffer, footer as fb};
 
-use crate::footer::segment::Segment;
+use crate::footer::segment::SegmentSpec;
 
 /// Captures the layout information of a Vortex file.
 pub(crate) struct Postscript {
-    pub(crate) dtype: Option<Segment>,
-    pub(crate) footer: Segment,
+    pub(crate) dtype: Option<SegmentSpec>,
+    pub(crate) footer: SegmentSpec,
 }
 
 impl FlatBufferRoot for Postscript {}
@@ -19,8 +19,8 @@ impl WriteFlatBuffer for Postscript {
         &self,
         fbb: &mut flatbuffers::FlatBufferBuilder<'fb>,
     ) -> flatbuffers::WIPOffset<Self::Target<'fb>> {
-        let dtype = self.dtype.as_ref().map(fb::Segment::from);
-        let footer = fb::Segment::from(&self.footer);
+        let dtype = self.dtype.as_ref().map(fb::SegmentSpec::from);
+        let footer = fb::SegmentSpec::from(&self.footer);
         fb::Postscript::create(
             fbb,
             &fb::PostscriptArgs {
@@ -39,8 +39,8 @@ impl ReadFlatBuffer for Postscript {
         fb: &<Self::Source<'buf> as Follow<'buf>>::Inner,
     ) -> Result<Self, Self::Error> {
         Ok(Self {
-            dtype: fb.dtype().map(Segment::try_from).transpose()?,
-            footer: Segment::try_from(
+            dtype: fb.dtype().map(SegmentSpec::try_from).transpose()?,
+            footer: SegmentSpec::try_from(
                 fb.footer()
                     .ok_or_else(|| vortex_err!("Postscript missing footer segment"))?,
             )?,
