@@ -2,14 +2,14 @@ use async_trait::async_trait;
 use moka::future::{Cache, CacheBuilder};
 use moka::policy::EvictionPolicy;
 use rustc_hash::FxBuildHasher;
-use vortex_buffer::{Alignment, ByteBuffer};
+use vortex_buffer::ByteBuffer;
 use vortex_error::{VortexExpect, VortexResult};
 use vortex_layout::segments::SegmentId;
 
 /// A cache for storing and retrieving individual segment data.
 #[async_trait]
 pub trait SegmentCache: Send + Sync {
-    async fn get(&self, id: SegmentId, alignment: Alignment) -> VortexResult<Option<ByteBuffer>>;
+    async fn get(&self, id: SegmentId) -> VortexResult<Option<ByteBuffer>>;
     async fn put(&self, id: SegmentId, buffer: ByteBuffer) -> VortexResult<()>;
     async fn remove(&self, id: SegmentId) -> VortexResult<()>;
 }
@@ -18,7 +18,7 @@ pub(crate) struct NoOpSegmentCache;
 
 #[async_trait]
 impl SegmentCache for NoOpSegmentCache {
-    async fn get(&self, _id: SegmentId, _alignment: Alignment) -> VortexResult<Option<ByteBuffer>> {
+    async fn get(&self, _id: SegmentId) -> VortexResult<Option<ByteBuffer>> {
         Ok(None)
     }
 
@@ -51,8 +51,8 @@ impl InMemorySegmentCache {
 
 #[async_trait]
 impl SegmentCache for InMemorySegmentCache {
-    async fn get(&self, id: SegmentId, alignment: Alignment) -> VortexResult<Option<ByteBuffer>> {
-        Ok(self.0.get(&id).await.map(|b| b.ensure_aligned(alignment)))
+    async fn get(&self, id: SegmentId) -> VortexResult<Option<ByteBuffer>> {
+        Ok(self.0.get(&id).await)
     }
 
     async fn put(&self, id: SegmentId, buffer: ByteBuffer) -> VortexResult<()> {
