@@ -12,7 +12,7 @@ use vortex_error::VortexResult;
 use crate::data::Layout;
 use crate::layouts::chunked::reader::ChunkedReader;
 use crate::reader::{LayoutReader, LayoutReaderExt};
-use crate::segments::AsyncSegmentReader;
+use crate::segments::SegmentSource;
 use crate::vtable::LayoutVTable;
 use crate::{CHUNKED_LAYOUT_ID, LayoutId};
 
@@ -20,9 +20,6 @@ use crate::{CHUNKED_LAYOUT_ID, LayoutId};
 pub struct ChunkedLayout;
 
 /// In-memory representation of Chunked layout.
-///
-/// First child in the list is the metadata table
-/// Subsequent children are consecutive chunks of this layout
 impl LayoutVTable for ChunkedLayout {
     fn id(&self) -> LayoutId {
         CHUNKED_LAYOUT_ID
@@ -31,10 +28,10 @@ impl LayoutVTable for ChunkedLayout {
     fn reader(
         &self,
         layout: Layout,
-        ctx: ArrayContext,
-        segment_reader: Arc<dyn AsyncSegmentReader>,
+        segment_source: &Arc<dyn SegmentSource>,
+        ctx: &ArrayContext,
     ) -> VortexResult<Arc<dyn LayoutReader>> {
-        Ok(ChunkedReader::try_new(layout, ctx, segment_reader)?.into_arc())
+        Ok(ChunkedReader::try_new(layout, segment_source.clone(), ctx.clone())?.into_arc())
     }
 
     fn register_splits(
@@ -44,9 +41,8 @@ impl LayoutVTable for ChunkedLayout {
         row_offset: u64,
         splits: &mut BTreeSet<u64>,
     ) -> VortexResult<()> {
-        let nchunks = layout.nchildren() - (if layout.metadata().is_some() { 1 } else { 0 });
         let mut offset = row_offset;
-        for i in 0..nchunks {
+        for i in 0..layout.nchildren() {
             let child = layout.child(i, layout.dtype().clone(), format!("[{}]", i))?;
             child.register_splits(field_mask, offset, splits)?;
             offset += child.row_count();
