@@ -1,48 +1,54 @@
-use std::sync::Arc;
-
+use vortex_array::arcref::ArcRef;
 use vortex_array::{ArrayContext, ArrayRef};
 use vortex_dtype::DType;
 use vortex_error::{VortexExpect, VortexResult};
 
-use crate::LayoutVTableRef;
 use crate::data::Layout;
 use crate::layouts::chunked::ChunkedLayout;
 use crate::layouts::flat::FlatLayout;
 use crate::segments::SegmentWriter;
 use crate::strategy::LayoutStrategy;
 use crate::writer::LayoutWriter;
+use crate::{LayoutVTableRef, LayoutWriterExt};
 
-pub struct ChunkedLayoutOptions {
+#[derive(Clone)]
+pub struct ChunkedLayoutStrategy {
     /// The layout strategy for each chunk.
-    pub chunk_strategy: Arc<dyn LayoutStrategy>,
+    pub chunk_strategy: ArcRef<dyn LayoutStrategy>,
 }
 
-impl Default for ChunkedLayoutOptions {
+impl Default for ChunkedLayoutStrategy {
     fn default() -> Self {
         Self {
-            chunk_strategy: Arc::new(FlatLayout),
+            chunk_strategy: ArcRef::new_ref(&FlatLayout),
         }
     }
 }
 
+impl LayoutStrategy for ChunkedLayoutStrategy {
+    fn new_writer(&self, ctx: &ArrayContext, dtype: &DType) -> VortexResult<Box<dyn LayoutWriter>> {
+        Ok(ChunkedLayoutWriter::new(ctx.clone(), dtype.clone(), self.clone()).boxed())
+    }
+}
+
 /// A basic implementation of a chunked layout writer that writes each batch into its own chunk.
 ///
 /// TODO(ngates): introduce more sophisticated layout writers with different chunking strategies.
 pub struct ChunkedLayoutWriter {
     ctx: ArrayContext,
-    options: ChunkedLayoutOptions,
+    options: ChunkedLayoutStrategy,
     chunks: Vec<Box<dyn LayoutWriter>>,
     dtype: DType,
     row_count: u64,
 }
 
 impl ChunkedLayoutWriter {
-    pub fn new(ctx: ArrayContext, dtype: &DType, options: ChunkedLayoutOptions) -> Self {
+    pub fn new(ctx: ArrayContext, dtype: DType, options: ChunkedLayoutStrategy) -> Self {
         Self {
             ctx,
             options,
-            chunks: Vec::new(),
-            dtype: dtype.clone(),
+            chunks: vec![],
+            dtype,
             row_count: 0,
         }
     }
@@ -51,7 +57,7 @@ impl ChunkedLayoutWriter {
 impl LayoutWriter for ChunkedLayoutWriter {
     fn push_chunk(
         &mut self,
-        segments: &mut dyn SegmentWriter,
+        segment_writer: &mut dyn SegmentWriter,
         chunk: ArrayRef,
     ) -> VortexResult<()> {
         self.row_count += chunk.len() as u64;
@@ -62,18 +68,24 @@ impl LayoutWriter for ChunkedLayoutWriter {
             .options
             .chunk_strategy
             .new_writer(&self.ctx, chunk.dtype())?;
-        chunk_writer.push_chunk(segments, chunk)?;
+        chunk_writer.push_chunk(segment_writer, chunk)?;
+        chunk_writer.flush(segment_writer)?;
         self.chunks.push(chunk_writer);
 
         Ok(())
     }
 
-    fn finish(&mut self, segments: &mut dyn SegmentWriter) -> VortexResult<Layout> {
+    fn flush(&mut self, _segment_writer: &mut dyn SegmentWriter) -> VortexResult<()> {
+        // We flush each chunk as we write it, so there's nothing to do here.
+        Ok(())
+    }
+
+    fn finish(&mut self, segment_writer: &mut dyn SegmentWriter) -> VortexResult<Layout> {
         // Call finish on each chunk's writer
         let mut children = vec![];
         for writer in self.chunks.iter_mut() {
             // FIXME(ngates): we should try calling finish after each chunk.
-            children.push(writer.finish(segments)?);
+            children.push(writer.finish(segment_writer)?);
         }
 
         // If there's only one child, there's no point even writing a stats table since
