@@ -64,7 +64,7 @@ fn update_stats(array: &dyn Array, stats: &[Stat]) -> VortexResult<()> {
 impl LayoutWriter for FlatLayoutWriter {
     fn push_chunk(
         &mut self,
-        segments: &mut dyn SegmentWriter,
+        segment_writer: &mut dyn SegmentWriter,
         chunk: ArrayRef,
     ) -> VortexResult<()> {
         if self.layout.is_some() {
@@ -80,7 +80,7 @@ impl LayoutWriter for FlatLayoutWriter {
                 include_padding: self.options.include_padding,
             },
         );
-        let segment_id = segments.put(&buffers);
+        let segment_id = segment_writer.put(&buffers);
 
         self.layout = Some(Layout::new_owned(
             "flat".into(),
@@ -94,7 +94,11 @@ impl LayoutWriter for FlatLayoutWriter {
         Ok(())
     }
 
-    fn finish(&mut self, _segments: &mut dyn SegmentWriter) -> VortexResult<Layout> {
+    fn flush(&mut self, _segment_writer: &mut dyn SegmentWriter) -> VortexResult<()> {
+        Ok(())
+    }
+
+    fn finish(&mut self, _segment_writer: &mut dyn SegmentWriter) -> VortexResult<Layout> {
         self.layout
             .take()
             .ok_or_else(|| vortex_err!("FlatLayoutStrategy::finish called without push_batch"))
@@ -112,10 +116,11 @@ mod tests {
     use vortex_array::{Array, ArrayContext};
     use vortex_buffer::buffer;
     use vortex_expr::ident;
+    use vortex_mask::Mask;
 
-    use crate::RowMask;
+    use crate::ExprEvaluator;
     use crate::layouts::flat::writer::FlatLayoutWriter;
-    use crate::segments::test::TestSegments;
+    use crate::segments::{SegmentSource, TestSegments};
     use crate::writer::LayoutWriterExt;
 
     // Currently, flat layouts do not force compute stats during write, they only retain
@@ -131,11 +136,14 @@ mod tests {
                 FlatLayoutWriter::new(ctx.clone(), array.dtype().clone(), Default::default())
                     .push_one(&mut segments, array.into_array())
                     .unwrap();
+            let segments: Arc<dyn SegmentSource> = Arc::new(segments);
 
             let result = layout
-                .reader(Arc::new(segments), ctx)
+                .reader(&segments, &ctx)
                 .unwrap()
-                .evaluate_expr(RowMask::new_valid_between(0, layout.row_count()), ident())
+                .projection_evaluation(&(0..layout.row_count()), &ident())
+                .unwrap()
+                .invoke(Mask::new_true(layout.row_count().try_into().unwrap()))
                 .await
                 .unwrap();
 
