@@ -216,6 +216,7 @@ impl Mask {
     }
 
     /// Create a new [`Mask`] from a [`Vec<usize>`].
+    // TODO(ngates): this should take an IntoIterator<usize>.
     pub fn from_indices(len: usize, indices: Vec<usize>) -> Self {
         let true_count = indices.len();
         assert!(indices.is_sorted(), "Mask indices must be sorted");
@@ -246,6 +247,28 @@ impl Mask {
         }))
     }
 
+    /// Create a new [`Mask`] from an [`IntoIterator<Item = usize>`] of indices to be excluded.
+    pub fn from_excluded_indices(len: usize, indices: impl IntoIterator<Item = usize>) -> Self {
+        let mut buf = BooleanBufferBuilder::new(len);
+        buf.append_n(len, true);
+
+        let mut false_count: usize = 0;
+        indices.into_iter().for_each(|idx| {
+            buf.set_bit(idx, false);
+            false_count += 1;
+        });
+        debug_assert_eq!(buf.len(), len);
+        let true_count = len - false_count;
+
+        Self::Values(Arc::new(MaskValues {
+            buffer: buf.finish(),
+            indices: Default::default(),
+            slices: Default::default(),
+            true_count,
+            density: true_count as f64 / len as f64,
+        }))
+    }
+
     /// Create a new [`Mask`] from a [`Vec<(usize, usize)>`] where each range
     /// represents a contiguous range of true values.
     pub fn from_slices(len: usize, vec: Vec<(usize, usize)>) -> Self {
@@ -512,6 +535,35 @@ impl FromIterator<bool> for Mask {
     }
 }
 
+impl FromIterator<Mask> for Mask {
+    fn from_iter<T: IntoIterator<Item = Mask>>(iter: T) -> Self {
+        let masks = iter.into_iter().collect::<Vec<_>>();
+        let total_length = masks.iter().map(|v| v.len()).sum();
+
+        // If they're all valid, then return a single validity.
+        if masks.iter().all(|v| v.all_true()) {
+            return Self::AllTrue(total_length);
+        }
+        // If they're all invalid, then return a single invalidity.
+        if masks.iter().all(|v| v.all_false()) {
+            return Self::AllFalse(total_length);
+        }
+
+        // Else, construct the boolean buffer
+        let mut buffer = BooleanBufferBuilder::new(total_length);
+        for mask in masks {
+            match mask {
+                Mask::AllTrue(count) => buffer.append_n(count, true),
+                Mask::AllFalse(count) => buffer.append_n(count, false),
+                Mask::Values(values) => {
+                    buffer.append_buffer(values.boolean_buffer());
+                }
+            };
+        }
+        Self::from_buffer(buffer.finish())
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
