@@ -2,7 +2,7 @@ use std::cmp::Ordering;
 
 use vortex_dtype::Nullability::NonNullable;
 use vortex_dtype::{DType, Nullability};
-use vortex_error::{vortex_bail, vortex_err, VortexError, VortexExpect as _, VortexResult};
+use vortex_error::{VortexError, VortexExpect as _, VortexResult, vortex_bail, vortex_err};
 
 use crate::{InnerScalarValue, Scalar, ScalarValue};
 
@@ -14,7 +14,7 @@ pub struct BoolScalar<'a> {
 
 impl PartialEq for BoolScalar<'_> {
     fn eq(&self, other: &Self) -> bool {
-        self.value == other.value
+        self.dtype.eq_ignore_nullability(other.dtype) && self.value == other.value
     }
 }
 
@@ -22,10 +22,13 @@ impl Eq for BoolScalar<'_> {}
 
 impl PartialOrd for BoolScalar<'_> {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        if self.dtype != other.dtype {
-            return None;
-        }
-        self.value.partial_cmp(&other.value)
+        Some(self.value.cmp(&other.value))
+    }
+}
+
+impl Ord for BoolScalar<'_> {
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.value.cmp(&other.value)
     }
 }
 
@@ -140,6 +143,8 @@ impl From<bool> for ScalarValue {
 
 #[cfg(test)]
 mod test {
+    use vortex_dtype::Nullability::*;
+
     use super::*;
 
     #[test]
@@ -147,4 +152,14 @@ mod test {
         let scalar: Scalar = false.into();
         assert!(!bool::try_from(&scalar).unwrap());
     }
+
+    #[test]
+    fn equality() {
+        assert_eq!(&Scalar::bool(true, Nullable), &Scalar::bool(true, Nullable));
+        // Equality ignores nullability
+        assert_eq!(
+            &Scalar::bool(true, Nullable),
+            &Scalar::bool(true, NonNullable)
+        );
+    }
 }
